"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
   Copyright 2017 Yuki KAN

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
const child_process_1 = require("child_process");
const latestVersion = require("latest-version");
const pkg = require("../package.json");
if (process.env.DOCKER === "YES") {
    console.error("Error: running in Docker.");
    process.exit(1);
}
if (process.platform !== "win32" && process.getuid() !== 0) {
    console.error("Error: root please.");
    process.exit(1);
}
if (!pkg._resolved) {
    console.error("Error: incompatible environment. (installed from not npm?)");
    process.exit(1);
}
(() => __awaiter(this, void 0, void 0, function* () {
    const current = pkg.version;
    console.log("current:", current);
    const latest = yield latestVersion("mirakurun");
    console.log("latest:", latest);
    if (current === latest) {
        console.log("already up to date.");
        process.exit(0);
    }
    if (current.split(".")[0] !== latest.split(".")[0]) {
        console.error("updater has aborted cause major version outdated.");
        process.exit(0);
    }
    console.log("updating...");
    const npm = spawnNpmInstall(latest);
    npm.on("exit", (code) => {
        if (code === 0) {
            console.log("updated successfully.");
            process.exit(0);
        }
        else {
            console.error("failed! reverting...");
            const npm = spawnNpmInstall(current);
            npm.on("exit", () => process.exit(1));
        }
    });
}))();
function spawnNpmInstall(version) {
    let command = "npm";
    const args = [
        "install",
        `${pkg.name}@${version}`,
        "-g",
        "--production"
    ];
    if (process.platform === "win32") {
        command = "npm.cmd";
    }
    else {
        args.push("--unsafe");
    }
    console.log(">", command, ...args);
    const npm = child_process_1.spawn(command, args, {
        stdio: ["ignore", process.stdout, process.stderr]
    });
    return npm;
}

//# sourceMappingURL=updater.js.map
